"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dummyModule = void 0;
const utils_1 = require("../utils");
const _1 = require(".");
const rust_binds_1 = require("../rust_binds");
const logger_1 = require("../utils/logger");
function versionCheck() {
    return true;
}
const initialize = () => {
    if (!versionCheck()) {
        return;
    }
    function compileRequest(_req, _res) {
        var _a;
        return {
            url: {
                host: (((_a = _1.METLO_META.opts) === null || _a === void 0 ? void 0 : _a.apiHost) || _req.socket.remoteAddress || "").toString(),
                path: (_req.baseUrl || "" + _req.path || "").toString(),
                parameters: Object.entries(_req.query).map(([k, v]) => ({
                    name: k.toString(),
                    value: v.toString(),
                })),
            },
            headers: Object.entries(_req.headers).map(([k, v]) => ({
                name: k.toString(),
                value: v.toString(),
            })),
            body: (typeof _req.body === "object"
                ? JSON.stringify(_req.body)
                : _req.body || "")
                .toString()
                .slice(0, _1.METLO_META.maxBodyCaptureSize),
            method: _req.method.toString(),
            user: _1.METLO_META.getUser
                ? _1.METLO_META.getUser(_req, _res)
                : null,
        };
    }
    function compileMeta(_req, _res) {
        return {
            environment: process.env.NODE_ENV || "development",
            incoming: true,
            source: _req.socket.remoteAddress.toString(),
            sourcePort: _req.socket.remotePort,
            destination: _req.socket.localAddress.toString(),
            destinationPort: _req.socket.localPort,
            metloSource: "node/express",
        };
    }
    function compileResponse(_req, _res, responseBody) {
        return {
            status: _res.statusCode,
            headers: Object.entries(_res.getHeaders()).map(([k, v]) => ({
                name: k.toString(),
                value: v.toString(),
            })),
            body: (typeof responseBody === "object"
                ? JSON.stringify(responseBody)
                : responseBody || "{}")
                .toString()
                .slice(0, _1.METLO_META.maxBodyCaptureSize),
        };
    }
    function compileInformation(_req, _res, responseBody) {
        try {
            const data = JSON.stringify({
                request: compileRequest(_req, _res),
                response: compileResponse(_req, _res, responseBody),
                meta: compileMeta(_req, _res),
            });
            (0, rust_binds_1.sendMetloTrace)(data);
        }
        catch (e) {
            (0, logger_1.Logger)().warn(`Metlo encountered an error ${e}`);
        }
    }
    if (_1.METLO_META.metloEnabled) {
        let middleware = (req, res, next) => {
            try {
                let shouldRejectChunks = false;
                const oldWrite = res.write;
                const oldEnd = res.end;
                const chunks = [];
                const chunkParse = (chunk) => {
                    try {
                        const currSize = chunks.reduce((acc, _ch) => acc + _ch.byteLength, 0);
                        if (typeof chunk === "string") {
                            if (chunk &&
                                _1.METLO_META.maxBodyCaptureSize - currSize > 0 &&
                                !shouldRejectChunks) {
                                (0, utils_1.addToMaxChunkSize)(chunks, Buffer.from(chunk), _1.METLO_META.maxBodyCaptureSize);
                            }
                            else {
                                shouldRejectChunks = true;
                            }
                        }
                        else if (chunk instanceof Buffer) {
                            if (chunk &&
                                _1.METLO_META.maxBodyCaptureSize - currSize > 0 &&
                                !shouldRejectChunks) {
                                if (chunk) {
                                    (0, utils_1.addToMaxChunkSize)(chunks, chunk, _1.METLO_META.maxBodyCaptureSize);
                                }
                            }
                            else {
                                shouldRejectChunks = true;
                            }
                        }
                    }
                    catch (err) {
                        (0, logger_1.Logger)().log("debug", err);
                    }
                };
                res.write = function (chunk) {
                    chunkParse(chunk);
                    return oldWrite.apply(res, arguments);
                };
                //@ts-ignore
                res.end = function (chunk) {
                    chunkParse(chunk);
                    oldEnd.apply(res, arguments);
                };
                res.on("finish", () => {
                    compileInformation(req, res, Buffer.concat(chunks).toString("utf8") || "{}");
                });
                if (!_1.METLO_META.disableBlocking) {
                    try {
                        const apiReq = JSON.stringify(compileRequest(req, res));
                        const apiMeta = JSON.stringify(compileMeta(req, res));
                        const willBlock = (0, rust_binds_1.shouldBlock)(apiReq, apiMeta);
                        if (willBlock) {
                            if (_1.METLO_META.rejectResponse) {
                                return _1.METLO_META.rejectResponse(req, res);
                            }
                            else {
                                return res.status(403).send();
                            }
                        }
                    }
                    catch (e) {
                        (0, logger_1.Logger)().warn(`Metlo encountered an error ${e}`);
                    }
                }
            }
            catch (err) {
                (0, logger_1.Logger)().log("debug", err);
            }
            return next();
        };
        return middleware;
    }
    else {
        return (0, exports.dummyModule)();
    }
};
const dummyModule = () => {
    return (req, res, next) => {
        next();
    };
};
exports.dummyModule = dummyModule;
exports.default = initialize;
//# sourceMappingURL=express.js.map