"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dummyModule = void 0;
const rust_binds_1 = require("../rust_binds");
const _1 = require(".");
const logger_1 = require("../utils/logger");
function versionCheck() {
    return true;
}
const initialize = () => {
    const fp = require("fastify-plugin");
    if (!versionCheck()) {
        return;
    }
    function compileRequest(request, response) {
        var _a;
        return {
            url: {
                host: (((_a = _1.METLO_META.opts) === null || _a === void 0 ? void 0 : _a.apiHost) || request.socket.remoteAddress || "").toString(),
                path: request.url.split("?")[0],
                parameters: Object.entries(request.query).map(([k, v]) => ({
                    name: k.toString(),
                    value: v.toString(),
                })),
            },
            headers: Object.entries(request.headers).map(([k, v]) => ({
                name: k.toString(),
                value: v.toString(),
            })),
            body: (typeof request.body === "object"
                ? JSON.stringify(request.body)
                : request.body || "{}")
                .toString()
                .slice(0, _1.METLO_META.maxBodyCaptureSize),
            method: request.method.toString(),
            user: _1.METLO_META.getUser
                ? _1.METLO_META.getUser(request, response)
                : null,
        };
    }
    function compileMeta(request, response) {
        return {
            environment: process.env.NODE_ENV || "development",
            incoming: true,
            source: request.socket.remoteAddress.toString(),
            sourcePort: request.socket.remotePort,
            destination: request.socket.localAddress.toString(),
            destinationPort: request.socket.localPort,
            metloSource: "node/fastify",
        };
    }
    function compileResponse(request, response, responseBody) {
        return {
            status: response.statusCode,
            headers: Object.entries(response.getHeaders()).map(([k, v]) => ({
                name: k.toString(),
                value: v.toString(),
            })),
            body: (typeof responseBody === "object"
                ? JSON.stringify(responseBody)
                : responseBody || "{}")
                .toString()
                .slice(0, _1.METLO_META.maxBodyCaptureSize),
        };
    }
    function compileInformation(request, response, responseBody) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const data = JSON.stringify({
                    request: compileRequest(request, response),
                    response: compileResponse(request, response, responseBody),
                    meta: compileMeta(request, response),
                });
                (0, rust_binds_1.sendMetloTrace)(data);
            }
            catch (e) {
                (0, logger_1.Logger)().warn(`Metlo encountered an error ${e}`);
            }
        });
    }
    if (_1.METLO_META.metloEnabled) {
        const callback = fp((fastify, options, done) => {
            try {
                fastify.addHook("preHandler", (req, res, next) => {
                    try {
                        if (!_1.METLO_META.disableBlocking) {
                            try {
                                const apiReq = JSON.stringify(compileRequest(req, res));
                                const apiMeta = JSON.stringify(compileMeta(req, res));
                                const willBlock = (0, rust_binds_1.shouldBlock)(apiReq, apiMeta);
                                if (willBlock) {
                                    if (_1.METLO_META.rejectResponse) {
                                        return _1.METLO_META.rejectResponse(req, res);
                                    }
                                    else {
                                        res.code(403);
                                        return res.send();
                                    }
                                }
                            }
                            catch (e) {
                                (0, logger_1.Logger)().warn(`Metlo encountered an error ${e}`);
                            }
                        }
                    }
                    catch (err) {
                        (0, logger_1.Logger)().log("debug", err);
                    }
                    return next();
                });
                fastify.addHook("onSend", (req, res, response, next) => {
                    try {
                        compileInformation(req, res, response);
                    }
                    catch (err) {
                        (0, logger_1.Logger)().log("debug", err);
                    }
                    return next();
                });
            }
            catch (err) {
                (0, logger_1.Logger)().log("debug", err);
            }
            return done();
        }, { name: "metlo", encapsulate: false });
        return callback;
    }
    else {
        return (0, exports.dummyModule)();
    }
};
const dummyModule = () => {
    const fp = require("fastify-plugin");
    return fp((fastify, options, done) => {
        done();
    });
};
exports.dummyModule = dummyModule;
exports.default = initialize;
//# sourceMappingURL=fastify.js.map