"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initMiddleware = exports.METLO_META = void 0;
const constants_1 = require("../utils/constants");
const rust_binds_1 = require("../rust_binds");
const logger_1 = require("../utils/logger");
const utils_1 = require("../utils");
exports.METLO_META = {
    key: "",
    host: "",
    backendPort: -1,
    collectorPort: -1,
    opts: {},
    maxBodyCaptureSize: 5 * 1024,
    metloEnabled: true,
    disableBlocking: false,
};
const initMiddleware = (props) => {
    let shouldMetloBeEnabled = true;
    exports.METLO_META.logLevel = props.logLevel || "info";
    (0, logger_1.Logger)(exports.METLO_META.logLevel);
    try {
        new URL(props.host);
    }
    catch (err) {
        shouldMetloBeEnabled = false;
        (0, logger_1.Logger)().log("error", `Metlo Host is not a proper url : ${props.host}`);
    }
    exports.METLO_META.host = props.host;
    if (typeof props.key === "string") {
        exports.METLO_META.key = props.key;
    }
    else {
        shouldMetloBeEnabled = false;
        (0, logger_1.Logger)().log("error", "Metlo API key passed is not a valid string.");
    }
    if (props.opts) {
        exports.METLO_META.opts = props.opts;
    }
    else {
        exports.METLO_META.opts = {};
    }
    if (props.backendPort) {
        if (typeof props.backendPort === "number" &&
            Number.isInteger(props.backendPort)) {
            exports.METLO_META.backendPort = props.backendPort;
        }
        else {
            shouldMetloBeEnabled = false;
            (0, logger_1.Logger)().log("error", "Metlo backend port passed is not a valid number.");
        }
    }
    else {
        exports.METLO_META.backendPort = props.host.includes("app.metlo.com") ? 443 : 8000;
    }
    if (props.collectorPort) {
        if (typeof props.collectorPort === "number" &&
            Number.isInteger(props.collectorPort)) {
            exports.METLO_META.collectorPort = props.collectorPort;
        }
        else {
            shouldMetloBeEnabled = false;
            (0, logger_1.Logger)().log("error", "Metlo backend port passed is not a valid number.");
        }
    }
    else {
        exports.METLO_META.collectorPort = 8081;
    }
    if (props.encryptionKey) {
        if (typeof props.encryptionKey === "string") {
            exports.METLO_META.encryptionKey = props.encryptionKey;
        }
        else {
            (0, logger_1.Logger)().log("error", "Metlo encryption key passed is not a valid string.");
        }
    }
    if (props.getUser) {
        if (typeof props.getUser === "function") {
            exports.METLO_META.getUser = props.getUser;
        }
        else {
            shouldMetloBeEnabled = false;
            (0, logger_1.Logger)().log("error", "getUser passed is not a valid function.");
        }
    }
    if (props.rejectResponse) {
        if (typeof props.rejectResponse === "function") {
            exports.METLO_META.rejectResponse = props.rejectResponse;
        }
        else {
            shouldMetloBeEnabled = false;
            (0, logger_1.Logger)().log("error", "rejectResponse passed is not a valid function.");
        }
    }
    if (props.maxBodyCaptureSize &&
        typeof props.maxBodyCaptureSize === "number" &&
        Number.isInteger(props.maxBodyCaptureSize)) {
        exports.METLO_META.maxBodyCaptureSize = props.maxBodyCaptureSize;
    }
    else {
        if (props.maxBodyCaptureSize) {
            if (typeof props.maxBodyCaptureSize !== "number") {
                (0, logger_1.Logger)().log("error", `maxBodyCaptureSize passed is not a valid number. Defaulting to size of ${constants_1.MAX_BODY_SIZE} bytes`);
            }
            else if (!Number.isInteger(props.maxBodyCaptureSize)) {
                (0, logger_1.Logger)().log("error", `maxBodyCaptureSize passed is not a valid integer. Defaulting to size of ${constants_1.MAX_BODY_SIZE} bytes`);
            }
        }
        exports.METLO_META.maxBodyCaptureSize = constants_1.MAX_BODY_SIZE;
    }
    let disableBlocking = false;
    if (typeof props.disableBlocking === "boolean") {
        disableBlocking = Boolean(props.disableBlocking);
        if (disableBlocking) {
            (0, logger_1.Logger)().log("debug", "Request Blocking by Metlo has been disabled");
        }
    }
    else {
        let flagValue = process.env.METLO_DISABLE_BLOCKING;
        if (flagValue !== undefined &&
            (flagValue === "true" || flagValue === "1" || flagValue === "yes")) {
            disableBlocking = true;
        }
    }
    exports.METLO_META.disableBlocking = disableBlocking;
    if (shouldMetloBeEnabled) {
        exports.METLO_META.metloEnabled = (0, utils_1.METLO_ENABLED)();
    }
    else {
        exports.METLO_META.metloEnabled = false;
    }
    if (exports.METLO_META.metloEnabled) {
        (0, rust_binds_1.startMetlo)(exports.METLO_META.host, exports.METLO_META.key, exports.METLO_META.backendPort, exports.METLO_META.collectorPort, exports.METLO_META.encryptionKey, exports.METLO_META.logLevel)
            .then(() => {
            (0, logger_1.Logger)().log("debug", "Metlo has started");
        })
            .catch(err => {
            (0, logger_1.Logger)().log("error", "Metlo encountered an error while initializing ");
            (0, logger_1.Logger)().log("error", err);
        });
    }
    else {
        (0, logger_1.Logger)().log("info", "Metlo is not enabled");
    }
};
exports.initMiddleware = initMiddleware;
exports.default = exports.initMiddleware;
//# sourceMappingURL=index.js.map