"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dummyModule = void 0;
const rust_binds_1 = require("../rust_binds");
const _1 = require(".");
const logger_1 = require("../utils/logger");
function versionCheck() {
    return true;
}
const initialize = () => {
    if (!versionCheck()) {
        return;
    }
    function compileRequest(ctx) {
        var _a;
        return {
            url: {
                host: (((_a = _1.METLO_META.opts) === null || _a === void 0 ? void 0 : _a.apiHost) || ctx.request.socket.remoteAddress || "").toString(),
                path: ctx.path || ctx.originalUrl || "",
                parameters: Object.entries(ctx.query || {}).map(([k, v]) => ({
                    name: k.toString(),
                    value: v.toString(),
                })),
            },
            headers: Object.entries(ctx.request.headers || {}).map(([k, v]) => ({
                name: k.toString(),
                value: v.toString(),
            })),
            body: (typeof ctx.request.body === "object"
                ? JSON.stringify(ctx.request.body)
                : ctx.request.body || "{}")
                .toString()
                .slice(0, _1.METLO_META.maxBodyCaptureSize),
            method: ctx.request.method.toString(),
            user: _1.METLO_META.getUser
                ? _1.METLO_META.getUser(ctx)
                : null,
        };
    }
    function compileMeta(ctx) {
        return {
            environment: process.env.NODE_ENV || "development",
            incoming: true,
            source: ctx.request.socket.remoteAddress.toString(),
            sourcePort: ctx.request.socket.remotePort,
            destination: ctx.request.socket.localAddress.toString(),
            destinationPort: ctx.request.socket.localPort,
            metloSource: "node/koa",
        };
    }
    function compileResponse(ctx) {
        return {
            status: ctx.response.status || 200,
            headers: Object.entries(ctx.response.headers || {}).map(([k, v]) => ({
                name: k.toString(),
                value: v.toString(),
            })),
            body: (typeof ctx.body === "object"
                ? JSON.stringify(ctx.body)
                : ctx.body || "{}")
                .toString()
                .slice(0, _1.METLO_META.maxBodyCaptureSize),
        };
    }
    function compileInformation(ctx) {
        try {
            const data = JSON.stringify({
                request: compileRequest(ctx),
                response: compileResponse(ctx),
                meta: compileMeta(ctx),
            });
            (0, rust_binds_1.sendMetloTrace)(data);
        }
        catch (e) {
            (0, logger_1.Logger)().warn(`Metlo encountered an error ${e}`);
        }
    }
    if (_1.METLO_META.metloEnabled) {
        const middleware = (ctx, next) => __awaiter(void 0, void 0, void 0, function* () {
            let rejectedResponse = false;
            try {
                if (!_1.METLO_META.disableBlocking) {
                    try {
                        const willBlock = (0, rust_binds_1.shouldBlock)(JSON.stringify(compileRequest(ctx)), JSON.stringify(compileMeta(ctx)));
                        if (willBlock) {
                            rejectedResponse = true;
                            if (_1.METLO_META.rejectResponse) {
                                // @ts-ignore
                                ;
                                _1.METLO_META.rejectResponse(ctx);
                            }
                            else {
                                ctx.status = 403;
                            }
                        }
                    }
                    catch (e) {
                        (0, logger_1.Logger)().warn(`Metlo encountered an error ${e}`);
                    }
                }
            }
            catch (err) {
                (0, logger_1.Logger)().log("debug", "Metlo encountered an error");
                (0, logger_1.Logger)().log("debug", err);
            }
            if (rejectedResponse) {
                // Don't do next if we're rejecting the response.
                compileInformation(ctx);
            }
            else {
                try {
                    yield next();
                    compileInformation(ctx);
                }
                catch (err) {
                    compileInformation(Object.assign(Object.assign({}, ctx), { response: Object.assign(Object.assign({}, ctx.response), { status: 500 }), body: ctx.response.body
                            ? ctx.response.body
                            : err
                                ? err.toString()
                                : "" }));
                    throw err;
                }
            }
        });
        return middleware;
    }
    else {
        return (0, exports.dummyModule)();
    }
};
const dummyModule = () => {
    return (ctx, next) => {
        next();
    };
};
exports.dummyModule = dummyModule;
exports.default = initialize;
//# sourceMappingURL=koa.js.map