"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.shouldBlock = exports.sendMetloTrace = exports.startMetlo = void 0;
const logger_1 = require("./utils/logger");
const detect_libc_1 = require("detect-libc");
let startup, parse_trace, block_trace, imports;
let enable = true;
let canStart = false;
let silenceNotStartWarnings = false;
if (process.platform === "linux") {
    if ((0, detect_libc_1.isNonGlibcLinuxSync)()) {
        imports = require("@metlo/metlo-node-agent-bindings-linux-x86-musl");
    }
    else {
        imports = require("@metlo/metlo-node-agent-bindings-linux-x86-gnu");
    }
}
else if (process.platform === "win32") {
    imports = require("@metlo/metlo-node-agent-bindings-windows");
}
else if (process.platform === "darwin") {
    if (process.arch === "x64") {
        imports = require("@metlo/metlo-node-agent-bindings-mac");
    }
    else if (process.arch === "arm64") {
        imports = require("@metlo/metlo-node-agent-bindings-mac-arm64");
    }
    else {
        (0, logger_1.Logger)().log("warn", `Metlo will not be enabled due to usage on unsupported architecture ${process.arch} on darwin.\nCurrently, metlo supports x86 and arm64 systems.`);
        enable = false;
    }
}
else {
    (0, logger_1.Logger)().log("warn", `Metlo will not be enabled due to usage on unsupported platform ${process.platform}.\nCurrently, metlo supports linux, windows and mac systems.`);
    enable = false;
}
startup = imports.startup;
parse_trace = imports.parse_trace;
block_trace = imports.block_trace;
const startMetlo = (metloHost, metloApiKey, backendPort, collectorPort, encryptionKey, logLevel) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        if (enable) {
            // First two params are required. The rest are optional.
            const params = [];
            params.push(metloHost);
            params.push(metloApiKey);
            if (collectorPort) {
                params.push(collectorPort);
            }
            else {
                params.push(null);
            }
            if (backendPort) {
                params.push(backendPort);
            }
            else {
                params.push(null);
            }
            if (encryptionKey) {
                params.push(encryptionKey);
            }
            else {
                params.push(null);
            }
            if (logLevel) {
                if (["info", "debug", "trace", "error", "warn"].includes(logLevel)) {
                    params.push(logLevel);
                }
                else {
                    params.push("info");
                }
            }
            else {
                params.push("info");
            }
            return new Promise((resolve, reject) => {
                try {
                    startup
                        .apply({}, params)
                        .then((response) => {
                        if (response) {
                            canStart = true;
                            resolve(true);
                        }
                        else {
                            if (typeof response === "boolean") {
                                reject("Could not initalize Metlo.");
                            }
                            else {
                                reject(response);
                            }
                        }
                    })
                        .catch(err => {
                        if (typeof err === "string") {
                            reject(err);
                        }
                        else {
                            reject(err & err.message ? err.message : null);
                        }
                    });
                }
                catch (err) {
                    if (typeof err === "string") {
                        reject(err);
                    }
                    else {
                        reject(err & err.message ? err.message : null);
                    }
                }
            });
        }
        return new Promise(() => false);
    }
    catch (err) {
        enable = false;
        (0, logger_1.Logger)().log("debug", err);
        return new Promise(() => false);
    }
});
exports.startMetlo = startMetlo;
const sendMetloTrace = trace => {
    try {
        if (canStart) {
            parse_trace(trace);
        }
        else {
            if (!silenceNotStartWarnings) {
                (0, logger_1.Logger)().log("debug", "Metlo did not start succesfully. Please check logs for more info.");
                silenceNotStartWarnings = true;
            }
        }
    }
    catch (err) {
        (0, logger_1.Logger)().log("debug", err);
    }
};
exports.sendMetloTrace = sendMetloTrace;
const shouldBlock = (requestTrace, requestMetadata) => {
    try {
        if (canStart) {
            return block_trace(requestTrace, requestMetadata);
        }
        else {
            if (!silenceNotStartWarnings) {
                (0, logger_1.Logger)().log("debug", "Metlo did not start succesfully. Please check logs for more info.");
                silenceNotStartWarnings = true;
            }
            return false;
        }
    }
    catch (err) {
        (0, logger_1.Logger)().log("debug", err);
        return false;
    }
};
exports.shouldBlock = shouldBlock;
//# sourceMappingURL=rust_binds.js.map