import { initMiddleware } from "./middlewares"
import ExpressModule, { MetloConfigExpress } from "./middlewares/express"
import KoaModule, { MetloConfigKoa } from "./middlewares/koa"
import FastifyModule, {
  MetloConfigFastify,
  fastifyRequest,
  fastifyResponse
} from "./middlewares/fastify"

import {
  Request as ExpressRequest,
  Response as ExpressResponse,
} from "express-serve-static-core"
import Application from "koa"

export const initFastify = <
  req extends fastifyRequest,
  res extends fastifyResponse,
>(
  params: MetloConfigFastify<req, res>,
) => {
  initMiddleware(params)
  return FastifyModule()
}

export const initExpress = <
  req extends ExpressRequest,
  res extends ExpressResponse,
>(
  params: MetloConfigExpress,
) => {
  initMiddleware(params)
  return ExpressModule<req, res>()
}

export const initKoa = <ctx extends Application.ExtendableContext>(
  params: MetloConfigKoa<ctx>,
) => {
  initMiddleware(params)
  return KoaModule<ctx>()
}
