import path from "node:path"
import fs from "fs"

const findClosestPackageJson = (dirPath: string) => {
  const filePath = path.join(dirPath, "package.json")
  if (!fs.existsSync(filePath)) {
    return findClosestPackageJson(dirPath.split("/").slice(0, -1).join("/"))
  } else {
    const data = JSON.parse(fs.readFileSync(filePath).toString("utf-8"))
    return data
  }
}

const findParentPackage = () => {
  const moduleParent = require.main
  return findClosestPackageJson(moduleParent.path)
}

export const getDependencies = () => {
  return Object.keys(findParentPackage().dependencies)
}

export const addToMaxChunkSize = (
  chunks: Array<Buffer>,
  newChunks: Buffer,
  maxSize: number,
) => {
  const existingSize = chunks.reduce(
    (acc, curr) => acc + Buffer.byteLength(curr),
    0,
  )
  if (existingSize < maxSize) {
    let subBuffer = newChunks.subarray(0, maxSize - existingSize)
    chunks.push(subBuffer)
  }
}

export const METLO_ENABLED = () => {
  return (
    process.env.METLO_ENABLE === undefined ||
    process.env.METLO_ENABLE === "true" ||
    process.env.METLO_ENABLE === "yes" ||
    process.env.METLO_ENABLE === "1"
  )
}
