import winston from "winston"

let _logger: winston.Logger = undefined

const metloFormat = winston.format.printf(
  ({ level, message, service, timestamp }) => {
    return `\[${timestamp} ${level} ${service}\]: ${message}`
  },
)

const levelMap = {
  trace: 4,
  debug: 3,
  info: 2,
  warn: 1,
  error: 0,
}

export const Logger = (level?: string) => {
  if (_logger) return _logger
  else {
    const _level = level ? level : "error"
    let a = winston.addColors({
      error: "red",
      debug: "blue",
      warn: "yellow",
      info: "green",
      trace: "magenta",
    })
    _logger = winston.createLogger({
      levels: levelMap,
      level: levelMap[level],
      format: winston.format.combine(
        winston.format(info => {
          info.level = info.level.toUpperCase()
          return info
        })(),
        winston.format.colorize(),
        winston.format.timestamp(),
        metloFormat,
      ),
      defaultMeta: { service: "metlo" },
      transports: [
        new winston.transports.Console({
          level: _level,
        } as winston.transports.ConsoleTransportOptions),
      ],
    })

    return _logger
  }
}
